@@ -1,9 +1,9 @@
 /* This file is part of GNU cflow
-   Copyright (C) 1997,2005 Sergey Poznyakoff
+   Copyright (C) 1997,2005,2007 Sergey Poznyakoff
  
    GNU cflow is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
  
    GNU cflow is distributed in the hope that it will be useful,
@@ -20,13 +20,13 @@
 #include <argp.h>
 #include <stdarg.h>
 #include <parser.h>
-#include <strndup.h>
+#include <version-etc.h>
 
 const char *argp_program_version = "cflow (" PACKAGE_NAME ") " VERSION;
 const char *argp_program_bug_address = "<" PACKAGE_BUGREPORT ">";
-/* TRANSLATORS: Please, preserve the vertical tabulation (^K character)
-   in this message */
-static char doc[] = N_("generate a program flowgraph\v* The effect of each option marked with an asterisk is reversed if the option's long name is prefixed with `no-'. For example, --no-cpp cancels --cpp.");
+static char doc[] = N_("generate a program flowgraph")
+"\v"
+N_("* The effect of each option marked with an asterisk is reversed if the option's long name is prefixed with `no-'. For example, --no-cpp cancels --cpp.");
 
 enum option_code {
      OPT_DEFINES = 256,
@@ -155,32 +155,12 @@ static struct argp_option options[] = {
        N_("* Verbose error diagnostics"), GROUP_ID+1 },
      { "no-verbose", OPT_NO_VERBOSE, NULL, OPTION_HIDDEN,
        "", GROUP_ID+1 },
-     { "license", 'L', 0, 0,
-       N_("Print license and exit"), GROUP_ID+1 },
      { "debug", OPT_DEBUG, "NUMBER", OPTION_ARG_OPTIONAL,
        N_("Set debugging level"), GROUP_ID+1 },
 #undef GROUP_ID     
      { 0, }
 };
 
-char *cflow_license_text = N_(
-"   GNU cflow is free software; you can redistribute it and/or modify\n"
-"   it under the terms of the GNU General Public License as published by\n"
-"   the Free Software Foundation; either version 2 of the License, or\n"
-"   (at your option) any later version.\n"
-"\n"
-"   GNU cflow is distributed in the hope that it will be useful,\n"
-"   but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-"   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-"   GNU General Public License for more details.\n"
-"\n"
-"   You should have received a copy of the GNU General Public License\n"
-"   along with GNU cflow; if not, write to the Free Software Foundation,\n"
-"   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA\n"
-"\n"
-"\n"
-);
-
 /* Structure representing various arguments of command line options */
 struct option_type {
     char *str;           /* optarg value */
@@ -214,11 +194,13 @@ int omit_symbol_names_option; /* Omit symbol name from symbol declaration string
 #define SM_STATIC      0x0004
 #define SM_UNDERSCORE  0x0008
 #define SM_TYPEDEF     0x0010
+#define SM_UNDEFINED   0x0020
 
 #define CHAR_TO_SM(c) ((c)=='x' ? SM_DATA : \
                         (c)=='_' ? SM_UNDERSCORE : \
                          (c)=='s' ? SM_STATIC : \
-                          (c)=='t' ? SM_TYPEDEF : 0)
+                          (c)=='t' ? SM_TYPEDEF : \
+                           (c)=='u' ? SM_UNDEFINED : 0)
 #define SYMBOL_INCLUDE(c) (symbol_map |= CHAR_TO_SM(c))
 #define SYMBOL_EXCLUDE(c) (symbol_map &= ~CHAR_TO_SM(c))
 int symbol_map;  /* A bitmap of symbols included in the graph. */
@@ -525,10 +507,6 @@ parse_opt (int key, char *arg, struct argp_state *state)
      case OPT_DEBUG:
 	  debug = arg ? atoi(arg) : 1;
 	  break;
-     case 'L':
-	  printf(_("License for %s:\n\n"), argp_program_version);
-	  printf("%s", cflow_license_text);
-	  exit(0);
      case 'P':
 	  set_print_option(arg);
 	  break;
@@ -593,6 +571,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
 	       case '_':
 	       case 's':
 	       case 't':
+	       case 'u':
 		    if (num)
 			 SYMBOL_INCLUDE(*arg);
 		    else
@@ -710,6 +689,10 @@ include_symbol(Symbol *sym)
 	       type |= SM_DATA;
 	  else if (sym->arity >= 0)
 	       type |= SM_FUNCTIONS;
+
+	  if (!sym->source)
+	       type |= SM_UNDEFINED;
+	  
      } else if (sym->type == SymToken) {
 	  if (sym->token_type == TYPE && sym->source)
 	       type |= SM_TYPEDEF;
@@ -741,11 +724,27 @@ init()
      init_parse();
 }
 
+const char version_etc_copyright[] =
+  /* Do *not* mark this string for translation.  %s is a copyright
+     symbol suitable for this locale, and %d is the copyright
+     year.  */
+  "Copyright %s 2005, 2006, %d Sergey Poznyakoff";
+
+static void
+cflow_version(FILE *stream, struct argp_state *state)
+{
+     	version_etc(stream, "cflow", PACKAGE_NAME, PACKAGE_VERSION,
+		    "Sergey Poznyakoff", NULL);
+}
+
 int
 main(int argc, char **argv)
 {
      int index;
 
+     program_name = argv[0]; /* Until gnulib provides a better way */
+     argp_program_version_hook = cflow_version;
+     
      setlocale(LC_ALL, "");
      bindtextdomain(PACKAGE, LOCALEDIR);
      textdomain(PACKAGE);
@@ -753,9 +752,9 @@ main(int argc, char **argv)
      register_output("gnu", gnu_output_handler, NULL);
      register_output("posix", posix_output_handler, NULL);
 
-     symbol_map = SM_FUNCTIONS|SM_STATIC;
+     symbol_map = SM_FUNCTIONS|SM_STATIC|SM_UNDEFINED;
 
-     if (getenv ("POSIXLY_CORRECT")) {
+     if (getenv("POSIXLY_CORRECT")) {
 	  if (select_output_driver("posix"))
 	       error(1, 0, _("%s: No such output driver"), "posix");
 	  output_init();
